package cz.toho.controller;

import cz.toho.exception.TodoListException;
import cz.toho.model.*;
import cz.toho.security.UserAuthentication;
import cz.toho.service.FileService;
import cz.toho.service.NoteService;
import cz.toho.service.TaskService;
import cz.toho.service.UserService;
import cz.toho.util.TodoConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.sql.SQLException;
import java.util.List;

/**
 * Rest controller for working with tasks
 * <p>
 * Created by tomasholicky
 */
@RestController
@RequestMapping("/tasks")
public class TaskController {
    /**
     * task service
     */
    @Autowired
    private TaskService taskService;
    /**
     * note service
     */
    @Autowired
    private NoteService noteService;
    /**
     * user service
     */
    @Autowired
    private UserService userService;

    @Autowired
    private FileService fileService;
    /**
     * logger
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(TaskController.class);

    /**
     * Rest mapping for load task by given task id
     *
     * @param idTask id task
     * @return {@link Task}
     */
    @Secured(TodoConstants.ROLE_USER)
    @RequestMapping(value = "/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Task> getTask(@PathVariable("id") Long idTask) {
        final UserAuthentication activeUser = (UserAuthentication) SecurityContextHolder.getContext().getAuthentication();
        if (activeUser != null) {
            try {
                final Task task = taskService.loadTask(activeUser.getName(), idTask);
                LOGGER.info("Loaded task with id: {}", idTask);
                return new ResponseEntity<>(task, HttpStatus.OK);
            } catch (TodoListException e) {
                LOGGER.error("Error while load task by given id: " + idTask, e);
                return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }

        return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
    }

    /**
     * Rest mapping for load task by given status
     *
     * @param status status
     * @return list of {@link Task}
     */
    @Secured(TodoConstants.ROLE_USER)
    @RequestMapping(method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Task>> getTasksByStatus(@RequestParam(value = "status", defaultValue = "TODO") String status) {
        final UserAuthentication activeUser = (UserAuthentication) SecurityContextHolder.getContext().getAuthentication();
        if (activeUser != null) {
            try {
                final List<Task> tasks = this.taskService.loadTasksByStatus(activeUser.getName(), TaskStatusEnum.valueOf(status));
                return new ResponseEntity<>(tasks, HttpStatus.OK);
            } catch (TodoListException e) {
                return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }

        return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
    }

    /**
     * Rest mapping for load task by given status
     *
     * @return list of {@link Task}
     */
    @Secured(TodoConstants.ROLE_USER)
    @RequestMapping(value = "/urgent", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Task>> getUrgentTasks() {
        final UserAuthentication activeUser = (UserAuthentication) SecurityContextHolder.getContext().getAuthentication();
        if (activeUser != null) {
            try {
                final List<Task> tasks = this.taskService.loadUrgentTasks(activeUser.getName());
                return new ResponseEntity<>(tasks, HttpStatus.OK);
            } catch (TodoListException e) {
                return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }

        return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
    }

    /**
     * Rest mapping for create task
     *
     * @param task task
     * @return id of created task
     */
    @Secured(TodoConstants.ROLE_USER)
    @RequestMapping(method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Long> addTask(@RequestBody Task task) {
        final UserAuthentication activeUser = (UserAuthentication) SecurityContextHolder.getContext().getAuthentication();
        if (activeUser != null) {
            try {
                final long idCreatedTask = this.taskService.addTask(task.getName(), task.getDescription(), activeUser.getDetails().getUsername());
                return new ResponseEntity<>(idCreatedTask, HttpStatus.CREATED);
            } catch (TodoListException e) {
                return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }

        return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
    }

    /**
     * Rest mapping for delete task
     *
     * @param idTask id of task
     * @return {@link HttpStatus}
     */
    @Secured(TodoConstants.ROLE_USER)
    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public HttpStatus deleteTask(@PathVariable("id") Long idTask) {
        final UserAuthentication activeUser = (UserAuthentication) SecurityContextHolder.getContext().getAuthentication();
        if (activeUser != null) {
            try {
                this.taskService.deleteTask(activeUser.getDetails().getUsername(), idTask);
                return HttpStatus.OK;
            } catch (TodoListException e) {
                return HttpStatus.INTERNAL_SERVER_ERROR;
            }
        } else {
            return HttpStatus.UNAUTHORIZED;
        }
    }

    /**
     * Rest mapping for update task
     *
     * @param idTask id of task
     * @param status status
     * @param urgent urgent
     * @param share  share
     * @return {@link HttpStatus}
     */
    @Secured(TodoConstants.ROLE_USER)
    @RequestMapping(value = "/{id}", method = RequestMethod.PUT)
    public HttpStatus updateTask(@PathVariable("id") Long idTask, @RequestParam(value = "status", required = false) String status, @RequestParam(value = "urgent", required = false) Boolean urgent, @RequestParam(value = "share", required = false) String share) {
        final UserAuthentication activeUser = (UserAuthentication) SecurityContextHolder.getContext().getAuthentication();
        if (activeUser != null) {
            try {
                if (status != null) {
                    this.taskService.updateTaskStatus(activeUser.getDetails().getUsername(), idTask, TaskStatusEnum.valueOf(status));
                }
                if (urgent != null) {
                    this.taskService.markUrgentTask(activeUser.getDetails().getUsername(), idTask, urgent);
                }
                if (share != null) {
                    this.taskService.shareTask(activeUser.getDetails().getUsername(), share, idTask);
                }
                return HttpStatus.OK;
            } catch (TodoListException e) {
                return HttpStatus.INTERNAL_SERVER_ERROR;
            }
        } else {
            return HttpStatus.UNAUTHORIZED;
        }
    }

    /**
     * Rest mapping for load list of note by by task
     *
     * @param idTask id of task
     * @return list of {@link Note}
     */
    @Secured(TodoConstants.ROLE_USER)
    @RequestMapping(value = "/{id}/notes", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Note>> getNotesByTask(@PathVariable("id") Long idTask) {
        final UserAuthentication activeUser = (UserAuthentication) SecurityContextHolder.getContext().getAuthentication();
        if (activeUser != null) {
            try {
                if (this.userService.checkRightForGivenTask(activeUser.getDetails().getUsername(), idTask)) {
                    final List<Note> notes = this.noteService.loadNotesByTask(activeUser.getDetails().getUsername(), idTask);
                    return new ResponseEntity<>(notes, HttpStatus.OK);
                } else {
                    return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
                }
            } catch (TodoListException e) {
                return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }
        return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
    }

    /**
     * Rest mapping for create note
     *
     * @param note   note
     * @param idTask id of task
     * @return id of created note
     */
    @Secured(TodoConstants.ROLE_USER)
    @RequestMapping(value = "/{id}/notes", method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Long> createNote(@RequestBody String note, @PathVariable("id") Long idTask) {
        final UserAuthentication activeUser = (UserAuthentication) SecurityContextHolder.getContext().getAuthentication();
        if (activeUser != null) {
            try {
                if (this.userService.checkRightForGivenTask(activeUser.getDetails().getUsername(), idTask)) {
                    final long idNote = this.noteService.createNote(activeUser.getDetails().getUsername(), idTask, note);
                    return new ResponseEntity<>(idNote, HttpStatus.CREATED);
                } else {
                    return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
                }
            } catch (TodoListException e) {
                return new ResponseEntity<Long>(HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }

        return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
    }

    /**
     * Rest mapping for create file
     *
     * @param file   file
     * @param idTask id of task
     * @return id of created file
     */
    @Secured(TodoConstants.ROLE_USER)
    @RequestMapping(value = "/{id}/files", method = RequestMethod.POST)
    public ResponseEntity<Long> createFile(@RequestParam("file") MultipartFile file, @PathVariable("id") Long idTask) {
        final UserAuthentication activeUser = (UserAuthentication) SecurityContextHolder.getContext().getAuthentication();
        if (activeUser != null) {
            try {
                if (this.userService.checkRightForGivenTask(activeUser.getDetails().getUsername(), idTask)) {
                    final long idFile = this.fileService.saveFile(activeUser.getDetails().getUsername(), idTask, file);
                    return new ResponseEntity<>(idFile, HttpStatus.CREATED);
                } else {
                    return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
                }
            } catch (TodoListException e) {
                return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }

        return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
    }

    /**
     * Rest mapping for load list of file info
     *
     * @param idTask id of task
     * @return list of file info
     */
    @Secured(TodoConstants.ROLE_USER)
    @RequestMapping(value = "/{id}/files", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<TaskFileInfo>> getFilesByTask(@PathVariable("id") Long idTask) {
        final UserAuthentication activeUser = (UserAuthentication) SecurityContextHolder.getContext().getAuthentication();
        if (activeUser != null) {
            try {
                if (this.userService.checkRightForGivenTask(activeUser.getDetails().getUsername(), idTask)) {
                    final List<TaskFileInfo> filesInfo = this.fileService.loadFilesInfo(idTask);
                    return new ResponseEntity<>(filesInfo, HttpStatus.OK);
                } else {
                    return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
                }
            } catch (TodoListException e) {
                return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }

        return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
    }

    /**
     * Rest mapping for load file by given task id and file id
     *
     * @param idTask id of task
     * @param idFile id of file
     * @return file
     */
    @Secured(TodoConstants.ROLE_USER)
    @RequestMapping(value = "/{idTask}/files/{idFile}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity getFile(@PathVariable("idTask") Long idTask, @PathVariable("idFile") Long idFile) {
        final UserAuthentication activeUser = (UserAuthentication) SecurityContextHolder.getContext().getAuthentication();
        if (activeUser != null) {
            try {
                if (this.userService.checkRightForGivenTask(activeUser.getDetails().getUsername(), idTask)) {
                    final TaskFile taskFile = this.fileService.loadFile(idFile, idTask);
                    if (taskFile != null) {
                        byte[] out = org.apache.commons.io.IOUtils.toByteArray(taskFile.getFile().getBinaryStream());

                        HttpHeaders responseHeaders = new HttpHeaders();
                        responseHeaders.add("todo-filename", taskFile.getFileName());
                        responseHeaders.setContentType(MediaType.parseMediaType(taskFile.getContentType()));

                        return new ResponseEntity<>(out, responseHeaders, HttpStatus.OK);
                    } else {
                        return new ResponseEntity<>(HttpStatus.NOT_FOUND);
                    }
                } else {
                    return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
                }
            } catch (TodoListException | IOException | SQLException e) {
                return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }
        return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
    }

    /**
     * Rest mapping for delete file
     *
     * @param idTask id of task
     * @param idFile id of file
     * @return {@link HttpStatus}
     */
    @Secured(TodoConstants.ROLE_USER)
    @RequestMapping(value = "/{idTask}/files/{idFiles}", method = RequestMethod.DELETE)
    public HttpStatus deleteFile(@PathVariable("idTask") Long idTask, @PathVariable("idFiles") Long idFile) {
        final UserAuthentication activeUser = (UserAuthentication) SecurityContextHolder.getContext().getAuthentication();
        if (activeUser != null) {
            try {
                this.fileService.deleteFile(idFile, idTask);
                return HttpStatus.OK;
            } catch (TodoListException e) {
                return HttpStatus.INTERNAL_SERVER_ERROR;
            }
        } else {
            return HttpStatus.UNAUTHORIZED;
        }
    }

}
