package cz.toho.dao;

import cz.toho.dao.mapper.TaskFileInfoMapper;
import cz.toho.dao.mapper.TaskFileMapper;
import cz.toho.exception.TodoListDaoException;
import cz.toho.model.TaskFileInfo;
import cz.toho.model.TaskFile;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Component;

import java.io.InputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;
import java.util.Properties;

/**
 * DAO for working with file
 * <p>
 * Created by tomasholicky
 */
@Component
public class FileDao {
    /**
     * jdbc template
     */
    @Autowired
    private JdbcTemplate todoListJdbcTemplate;
    /**
     * list of queries
     */
    @Autowired
    private Properties todoListQueryProps;
    /**
     * file info mapper
     */
    @Autowired
    private TaskFileInfoMapper taskFileInfoMapper;
    /**
     * file mapper
     */
    @Autowired
    private TaskFileMapper taskFileMapper;
    /**
     * logger
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(FileDao.class);

    /**
     * Create file
     *
     * @param idTask      id of task
     * @param idUser      id of user
     * @param filename    file name
     * @param contentType content type
     * @param fileStream  file stream
     * @return id of created file
     * @throws TodoListDaoException
     */
    public long createFile(final long idTask, final long idUser, final String filename, final String contentType, final InputStream fileStream) throws TodoListDaoException {
        KeyHolder keyHolder = new GeneratedKeyHolder();

        try {
            final String sqlQuery = todoListQueryProps.getProperty("saveFile");

            this.todoListJdbcTemplate.update(new PreparedStatementCreator() {
                @Override
                public PreparedStatement createPreparedStatement(Connection connection) throws SQLException {
                    PreparedStatement ps = connection.prepareStatement(sqlQuery, Statement.RETURN_GENERATED_KEYS);
                    ps.setLong(1, idTask);
                    ps.setLong(2, idUser);
                    ps.setString(3, filename);
                    ps.setBlob(4, fileStream);
                    ps.setString(5, contentType);
                    return ps;
                }
            }, keyHolder);
        } catch (Exception e) {
            LOGGER.error("Error while creating file", e);
            throw new TodoListDaoException(e);
        }

        long createdId = keyHolder.getKey().longValue();
        LOGGER.info("Created file with id: {}", createdId);

        return createdId;
    }

    /**
     * Load list of file info, by given id task
     *
     * @param idTask id of task
     * @return list of file info
     * @throws TodoListDaoException
     */
    public List<TaskFileInfo> getFilesInfoByTask(final long idTask) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("getFilesInfoByTask");
            return this.todoListJdbcTemplate.query(sqlQuery, this.taskFileInfoMapper, idTask);
        } catch (Exception e) {
            LOGGER.error("Error while loading list of file info", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Load file, by given id file and id task
     *
     * @param idFile id of file
     * @param idTask id of task
     * @return file
     * @throws TodoListDaoException
     */
    public TaskFile loadFile(final long idFile, final long idTask) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("loadFile");
            return this.todoListJdbcTemplate.queryForObject(sqlQuery, this.taskFileMapper, idFile, idTask);
        } catch (Exception e) {
            LOGGER.error("Error while loading file", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Delete file
     *
     * @param idFile id of file
     * @param idTask id of task
     * @throws TodoListDaoException
     */
    public void deleteFile(final Long idFile, final Long idTask) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("deleteFile");
            this.todoListJdbcTemplate.update(sqlQuery, idFile, idTask);
            LOGGER.info("Deleted file wiht id: {}", idFile);
        } catch (Exception e) {
            LOGGER.error("Error while creating file", e);
            throw new TodoListDaoException(e);
        }
    }
}
