package cz.toho.dao;

import cz.toho.dao.mapper.NoteMapper;
import cz.toho.exception.TodoListDaoException;
import cz.toho.model.Note;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Component;

import java.sql.*;
import java.util.List;
import java.util.Properties;

/**
 * DAO for working with note
 * <p>
 * Created by tomasholicky
 */
@Component
public class NoteDao {
    /**
     * jdbc template
     */
    @Autowired
    private JdbcTemplate todoListJdbcTemplate;
    /**
     * note mapper
     */
    @Autowired
    private NoteMapper noteMapper;
    /**
     * list of queries
     */
    @Autowired
    private Properties todoListQueryProps;
    /**
     * logger
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(NoteDao.class);

    /**
     * Load list of note by id task
     *
     * @param idTask id of task
     * @return list of {@link Note}
     * @throws TodoListDaoException
     */
    public List<Note> getNotesByTask(final long idTask) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("getNotesByTask");
            return this.todoListJdbcTemplate.query(sqlQuery, this.noteMapper, idTask);
        } catch (Exception e) {
            LOGGER.error("Error while loading notes by id task", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Create note
     *
     * @param idTask id of task
     * @param idUser id of user
     * @param note   note
     * @return created id of note
     * @throws TodoListDaoException
     */
    public long createNote(final long idTask, final long idUser, final String note) throws TodoListDaoException {
        KeyHolder keyHolder = new GeneratedKeyHolder();

        final java.util.Date effectiveDate = new java.util.Date();
        try {
            final String sqlQuery = todoListQueryProps.getProperty("createNote");

            this.todoListJdbcTemplate.update(new PreparedStatementCreator() {
                @Override
                public PreparedStatement createPreparedStatement(Connection connection) throws SQLException {
                    PreparedStatement ps = connection.prepareStatement(sqlQuery, Statement.RETURN_GENERATED_KEYS);
                    ps.setLong(1, idTask);
                    ps.setLong(2, idUser);
                    ps.setString(3, note);
                    ps.setTimestamp(4, new Timestamp(effectiveDate.getTime()));
                    return ps;
                }
            }, keyHolder);
        } catch (Exception e) {
            LOGGER.error("Error while creating note", e);
            throw new TodoListDaoException(e);
        }

        long createdId = keyHolder.getKey().longValue();
        LOGGER.info("Created file with id: {}", createdId);

        return createdId;
    }
}
