package cz.toho.dao;

import cz.toho.dao.mapper.TaskMapper;
import cz.toho.exception.TodoListDaoException;
import cz.toho.model.Task;
import cz.toho.model.TaskStatusEnum;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Component;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;
import java.util.Properties;

/**
 * DAO for working with task
 * <p>
 * Created by tomasholicky
 */
@Component
public class TaskDao {
    /**
     * jdbc template
     */
    @Autowired
    private JdbcTemplate todoListJdbcTemplate;
    /**
     * task mapper
     */
    @Autowired
    private TaskMapper taskMapper;
    /**
     * list of queries
     */
    @Autowired
    private Properties todoListQueryProps;
    /**
     * logger
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(TaskDao.class);

    /**
     * Load task by given id task and id user
     *
     * @param idTask id of task
     * @param idUser id of user
     * @return {@link Task}
     * @throws TodoListDaoException
     */
    public Task getTask(final Long idTask, final Long idUser) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("getTask");
            return this.todoListJdbcTemplate.queryForObject(sqlQuery, taskMapper, idTask, idUser);
        } catch (Exception e) {
            LOGGER.error("Error while loading task", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Load list of task, by given user id and task status
     *
     * @param userId     id of user
     * @param taskStatus status of task
     * @return list of {@link Task}
     * @throws TodoListDaoException
     */
    public List<Task> getTasksByStatus(final long userId, final TaskStatusEnum taskStatus) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("getTasksByStatus");
            return this.todoListJdbcTemplate.query(sqlQuery, taskMapper, userId, taskStatus.toString());
        } catch (Exception e) {
            LOGGER.error("Error while loading list of task, by status", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Load urgent task
     *
     * @param userId id of user
     * @return list of {@link Task}
     * @throws TodoListDaoException
     */
    public List<Task> getUrgentTasks(final long userId) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("getUrgentTasks");
            return this.todoListJdbcTemplate.query(sqlQuery, taskMapper, userId);
        } catch (Exception e) {
            LOGGER.error("Error while loading list of urgent task", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Add task
     *
     * @param name        name
     * @param description description
     * @param taskStatus  task status
     * @return created id of task
     * @throws TodoListDaoException
     */
    public long addTask(final String name, final String description, final TaskStatusEnum taskStatus) throws TodoListDaoException {
        KeyHolder keyHolder = new GeneratedKeyHolder();

        try {
            final String sqlQuery = todoListQueryProps.getProperty("addTask");

            this.todoListJdbcTemplate.update(new PreparedStatementCreator() {
                @Override
                public PreparedStatement createPreparedStatement(Connection connection) throws SQLException {
                    PreparedStatement ps = connection.prepareStatement(sqlQuery, Statement.RETURN_GENERATED_KEYS);
                    ps.setString(1, name);
                    ps.setString(2, description);
                    ps.setString(3, taskStatus.toString());
                    return ps;
                }
            }, keyHolder);
        } catch (Exception e) {
            LOGGER.error("Error while creating task", e);
            throw new TodoListDaoException(e);
        }

        long createdId = keyHolder.getKey().longValue();
        LOGGER.info("Created file with id: {}", createdId);

        return createdId;
    }

    /**
     * Add task relationship
     *
     * @param idUser id of user
     * @param idTask id of task
     * @throws TodoListDaoException
     */
    public void addTaskRelationship(final long idUser, final long idTask) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("addTaskRelationship");
            this.todoListJdbcTemplate.update(sqlQuery, idUser, idTask, 0);
            LOGGER.info("Added relationship of task between idUser: {} and idTask: {}", idUser, idTask);
        } catch (Exception e) {
            LOGGER.error("Error while loading adding relationship of task", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Update task status
     *
     * @param taskStatus task status
     * @param idTask     id of task
     * @param idUser     id of user
     * @throws TodoListDaoException
     */
    public void updateTaskStatus(final TaskStatusEnum taskStatus, final long idTask, final long idUser) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("updateTaskStatus");
            this.todoListJdbcTemplate.update(sqlQuery, taskStatus.toString(), idTask, idUser);
            LOGGER.info("Task with id: {} updated status on: {}", idTask, taskStatus.name());
        } catch (Exception e) {
            LOGGER.error("Error while updating status of task", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Mark task as urgent
     *
     * @param urgent urgent
     * @param idTask id of task
     * @param idUser id of user
     * @throws TodoListDaoException
     */
    public void markUrgentTask(final boolean urgent, final long idTask, final long idUser) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("markUrgentTask");
            this.todoListJdbcTemplate.update(sqlQuery, urgent, idTask, idUser);
            LOGGER.info("Task with id: {} is marked as urgent", idTask);
        } catch (Exception e) {
            LOGGER.error("Error while marking task as urgent", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Check relationship between task and user
     *
     * @param idTask id of task
     * @param idUser id of user
     * @return boolean
     * @throws TodoListDaoException
     */
    public boolean checkRelationship(final long idTask, final long idUser) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("checkTaskRelationship");
            int count = this.todoListJdbcTemplate.queryForObject(sqlQuery, new Object[]{idTask, idUser}, Integer.class);
            return count > 0;
        } catch (Exception e) {
            LOGGER.error("Error while checking relationship between task and user", e);
            throw new TodoListDaoException(e);
        }
    }
}
