package cz.toho.dao;

import cz.toho.dao.mapper.RoleMapper;
import cz.toho.dao.mapper.UserMapper;
import cz.toho.exception.TodoListDaoException;
import cz.toho.model.Role;
import cz.toho.model.User;
import cz.toho.util.UserRole;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Component;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;
import java.util.Properties;

/**
 * DAO for working with user
 * <p>
 * Created by tomasholicky
 */
@Component
public class UserDao {
    /**
     * jdbc template
     */
    @Autowired
    private JdbcTemplate todoListJdbcTemplate;
    /**
     * user mapper
     */
    @Autowired
    private UserMapper userMapper;
    /**
     * role mapper
     */
    @Autowired
    private RoleMapper roleMapper;
    /**
     * list of queries
     */
    @Autowired
    private Properties todoListQueryProps;
    /**
     * password encoder
     */
    @Autowired
    private BCryptPasswordEncoder passwordEncoder;
    /**
     * logger
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(UserDao.class);

    /**
     * Load user by given user name
     *
     * @param username user name
     * @return {@link User}
     * @throws TodoListDaoException
     */
    public User getUser(final String username) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("getUserByUsername");
            return this.todoListJdbcTemplate.queryForObject(sqlQuery, this.userMapper, username);
        } catch (Exception e) {
            LOGGER.error("Error while loading user", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Load list of rule by given user id
     *
     * @param userId id of user
     * @return list of {@link Role}
     * @throws TodoListDaoException
     */
    public List<Role> getUserRoles(final Long userId) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("rolesByUserId");
            return this.todoListJdbcTemplate.query(sqlQuery, this.roleMapper, userId);
        } catch (Exception e) {
            LOGGER.error("Error while loading user role", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Create user
     *
     * @param userName user name
     * @param password password
     * @return created id of user
     * @throws TodoListDaoException
     */
    public long createUser(final String userName, final String password) throws TodoListDaoException {
        KeyHolder keyHolder = new GeneratedKeyHolder();
        try {
            final String sqlQuery = todoListQueryProps.getProperty("createUser");

            this.todoListJdbcTemplate.update(new PreparedStatementCreator() {
                @Override
                public PreparedStatement createPreparedStatement(Connection connection) throws SQLException {
                    PreparedStatement ps = connection.prepareStatement(sqlQuery, Statement.RETURN_GENERATED_KEYS);
                    ps.setString(1, userName);
                    ps.setString(2, passwordEncoder.encode(password));
                    return ps;
                }
            }, keyHolder);
        } catch (Exception e) {
            LOGGER.error("Error while creating user", e);
            throw new TodoListDaoException(e);
        }

        long createdId = keyHolder.getKey().longValue();
        LOGGER.info("Created user with id: {}", createdId);

        return createdId;
    }

    /**
     * Create user role
     *
     * @param idUser id fo user
     * @throws TodoListDaoException
     */
    public void createUserRole(final long idUser) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("createUserRole");
            this.todoListJdbcTemplate.update(sqlQuery, idUser, UserRole.ROLE_USER.getIdRole());
        } catch (Exception e) {
            LOGGER.error("Error while creating user role", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Check, that exists user
     *
     * @param userName user name
     * @return int
     * @throws TodoListDaoException
     */
    public int existsUser(final String userName) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("existsUser");
            return this.todoListJdbcTemplate.queryForObject(sqlQuery, new Object[]{userName}, Integer.class);
        } catch (Exception e) {
            LOGGER.error("Error while checking, that exists user", e);
            throw new TodoListDaoException(e);
        }
    }

    /**
     * Check right for given task
     *
     * @param idUser id of user
     * @param idTask id of task
     * @return boolean
     * @throws TodoListDaoException
     */
    public boolean checkRightForGivenTask(final long idUser, final long idTask) throws TodoListDaoException {
        try {
            final String sqlQuery = todoListQueryProps.getProperty("checkRightForGivenTask");
            int count = this.todoListJdbcTemplate.queryForObject(sqlQuery, new Object[]{idTask, idUser}, Integer.class);
            return count > 0;
        } catch (Exception e) {
            LOGGER.error("Error while checking right for given task", e);
            throw new TodoListDaoException(e);
        }
    }
}
