package cz.toho.security;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.Authentication;
import org.springframework.security.jwt.Jwt;
import org.springframework.security.jwt.JwtHelper;
import org.springframework.security.jwt.crypto.sign.MacSigner;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Date;

/**
 * Service for working with token
 * <p>
 * Created by tomasholicky
 */

@Service
public class TokenAuthenticationService {
    /**
     * header name for token
     */
    private static final String AUTH_HEADER_NAME = "X-Auth-Token";
    /**
     * word for secret
     */
    private MacSigner signer;
    /**
     * expires
     */
    private int expires;

    @Autowired
    public TokenAuthenticationService(@Value("${token.secret}") String secret, @Value("${token.expires}") int expires) {
        this.signer = new MacSigner(secret);
        this.expires = expires;
    }

    /**
     * Adds authentication header to the response
     * <p>
     * Calculates JWT token from user information and adds expiration limit
     *
     * @param response       response
     * @param authentication authentication
     */
    public void addAuthenticationHeader(HttpServletResponse response, Authentication authentication) {
        if (authentication == null) {
            response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
        } else {
            final UserClaims userClaims = new UserClaims(authentication, expires);
            Jwt token = JwtHelper.encode(toJSON(userClaims), this.signer);
            response.addHeader(AUTH_HEADER_NAME, token.getEncoded());
        }
    }

    /**
     * User claim to JSON object
     *
     * @param authentication authentication
     * @return JSON
     */
    private String toJSON(UserClaims authentication) {
        try {
            return new ObjectMapper().writeValueAsString(authentication);
        } catch (JsonProcessingException e) {
            throw new IllegalStateException(e);
        }
    }

    /**
     * Read authentication header and parse JWT token from it.
     * Checks if token is not expired
     *
     * @param request request
     * @return User which is being authenticated or NULL
     */
    public Authentication getAuthentication(HttpServletRequest request) {

        final String token = request.getHeader(AUTH_HEADER_NAME);

        if (token == null) {
            return null;
        }

        final Jwt jwt = JwtHelper.decodeAndVerify(token, this.signer);
        String json = jwt.getClaims();
        UserClaims userClaims = fromJSON(json);

        final long timestamp = new Date().getTime() / 1000;
        if (userClaims.getExp() <= timestamp) {
            return null;
        }

        return new UserAuthentication(userClaims);
    }

    /**
     * JSON object to user claim
     *
     * @param json json
     * @return {@link UserClaims}
     */
    private UserClaims fromJSON(String json) {
        try {
            return new ObjectMapper().readValue(json, UserClaims.class);
        } catch (IOException e) {
            throw new IllegalStateException(e);
        }
    }
}
