package cz.toho.service.impl;

import cz.toho.dao.TaskDao;
import cz.toho.dao.UserDao;
import cz.toho.exception.TodoListDaoException;
import cz.toho.exception.TodoListException;
import cz.toho.model.Task;
import cz.toho.model.TaskStatusEnum;
import cz.toho.model.User;
import cz.toho.service.TaskService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * Implementation of task service
 * <p>
 * Created by tomasholicky
 */
@Service
@Transactional("todoListTransactionManager")
public class TaskServiceImpl implements TaskService {
    /**
     * task dao
     */
    @Autowired
    private TaskDao taskDao;
    /**
     * user dao
     */
    @Autowired
    private UserDao userDao;

    @Override
    public Task loadTask(final String userName, final Long idTask) throws TodoListException {
        try {
            final User user = this.userDao.getUser(userName);
            return taskDao.getTask(idTask, user.getUserId());
        } catch (TodoListDaoException e) {
            throw new TodoListException(e);
        }
    }

    @Override
    public List<Task> loadTasksByStatus(final String userName, final TaskStatusEnum taskStatus) throws TodoListException {
        try {
            final User user = this.userDao.getUser(userName);
            return this.taskDao.getTasksByStatus(user.getUserId(), taskStatus);
        } catch (TodoListDaoException e) {
            throw new TodoListException(e);
        }
    }

    @Override
    public List<Task> loadUrgentTasks(String userName) throws TodoListException {
        try {
            final User user = this.userDao.getUser(userName);
            return this.taskDao.getUrgentTasks(user.getUserId());
        } catch (TodoListDaoException e) {
            throw new TodoListException(e);
        }
    }

    @Override
    public long addTask(final String name, final String description, final String userName) throws TodoListException {
        try {
            final User user = this.userDao.getUser(userName);
            final long createdTaskId = this.taskDao.addTask(name, description, TaskStatusEnum.TODO);
            this.taskDao.addTaskRelationship(user.getUserId(), createdTaskId);
            return createdTaskId;
        } catch (TodoListDaoException e) {
            throw new TodoListException(e);
        }
    }

    @Override
    public void deleteTask(final String userName, final long idTask) throws TodoListException {
        try {
            final User user = this.userDao.getUser(userName);
            this.taskDao.updateTaskStatus(TaskStatusEnum.CANCELED, idTask, user.getUserId());
        } catch (TodoListDaoException e) {
            throw new TodoListException(e);
        }
    }

    @Override
    public void updateTaskStatus(final String userName, final long idTask, final TaskStatusEnum taskStatus) throws TodoListException {
        try {
            final User user = this.userDao.getUser(userName);
            this.taskDao.updateTaskStatus(taskStatus, idTask, user.getUserId());
        } catch (TodoListDaoException e) {
            throw new TodoListException(e);
        }
    }

    @Override
    public void markUrgentTask(String userName, long idTask, boolean urgent) throws TodoListException {
        try {
            final User user = this.userDao.getUser(userName);
            this.taskDao.markUrgentTask(urgent, idTask, user.getUserId());
        } catch (TodoListDaoException e) {
            throw new TodoListException(e);
        }
    }

    @Override
    public void shareTask(String currentUserName, String shareName, long idTask) throws TodoListException {
        try {
            final User currentUser = this.userDao.getUser(currentUserName);
            final User shareUser = this.userDao.getUser(shareName);
            if (this.taskDao.checkRelationship(idTask, currentUser.getUserId())) {
                this.taskDao.addTaskRelationship(shareUser.getUserId(), idTask);
            }
        } catch (TodoListDaoException e) {
            throw new TodoListException(e);
        }
    }
}
