/**
 * Created by tomasholicky
 */
(function () {
    'use strict';

    var AuthServiceSettingsProvider = function () {
        var setting = {
            urls: {
                login: 'login',
                tokenRenew: 'renew'
            },
            tokenHeaderName: 'X-Auth-Token'
        };

        this.setLoginUrl = function (url) {
            setting.urls.login = url;
        };

        this.setTokenRenewUrl = function (url) {
            setting.urls.tokenRenew = url;
        };

        this.$get = function () {
            return setting;
        };
    };

    var authServiceFactory = function ($http, $rootScope, $q, authTokenStore, $state, $location, authServiceSettings, EVENTS) {

        var successLogin = function (response) {
            auth.setToken(response.headers(authServiceSettings.tokenHeaderName));

            $rootScope.$broadcast(EVENTS.loginSuccess);
            $state.go('user.tasks');

            return response;
        };

        var failedLogin = function (response) {
            $rootScope.$broadcast(EVENTS.loginFailed);
            return $q.reject(response);
        };

        var renewToken = function (response) {
            auth.setToken(response.headers(authServiceSettings.tokenHeaderName));
        };

        var auth = {
            info: authTokenStore.getInfo(),

            getUserInfo: function () {
                return this.info.user;
            },

            login: function (name, pass) {
                return $http.post(authServiceSettings.urls.login, {username: name, password: pass})
                    .then(successLogin, failedLogin);
            },

            logout: function () {
                this.clearAuth();
                $location.path('/');
            },

            renewToken: function () {
                return $http.post(authServiceSettings.urls.tokenRenew, {})
                    .then(renewToken);
            },

            setToken: function (token) {
                if (token === null) {
                    this.clearAuth();
                    return;
                }

                authTokenStore.setToken(token);
            },

            clearAuth: function () {
                authTokenStore.clear();
            },

            isAuthenticated: function () {
                return Boolean(this.info.user) || false;
            },

            hasRole: function (role) {
                if (!this.info.user) {
                    return false;
                }

                if (!this.info.user.roles) {
                    return false;
                }

                return this.info.user.roles.indexOf(role) !== -1;
            },

            hasSomeRole: function (roles) {
                if (!this.isAuthenticated()) {
                    return false;
                }

                return roles.some(this.hasRole.bind(this));
            }
        };

        return auth;

    };

    var AuthTokenStore = function (jwtHelper, webStorage, $rootScope, EVENTS) {

        var TOKEN_STORE_KEY = 'cz.toho.auth:token';

        this.info = {};

        /**
         * @returns {{user: {name: string}, roles: Array}}
         */
        this.getInfo = function () {
            return this.info;
        };

        this.clear = function () {
            delete this.info.token;
            delete this.info.user;
            delete this.info.expires;

            webStorage.session.remove(TOKEN_STORE_KEY);

            $rootScope.$broadcast(EVENTS.changedState);
        };

        /**
         * @param token
         */
        this.setToken = function (token) {
            if (!token) {
                return;
            }

            webStorage.session.add('cz.toho.auth:token', token);

            this.info.token = token;
            this.info.user = jwtHelper.decodeToken(token);
            this.info.expires = jwtHelper.getTokenExpirationDate(token);

            $rootScope.$broadcast(EVENTS.changedState);
        };

        this.setToken(webStorage.session.get(TOKEN_STORE_KEY));
    };

    angular.module('cz.toho.auth.service', ['angular-jwt', 'webStorageModule'])
        .provider('authServiceSettings', AuthServiceSettingsProvider)
        .factory('authService', authServiceFactory)
        .service('authTokenStore', AuthTokenStore);
})();